/* eslint-disable no-magic-numbers */
import {
  ToolkitStoreWithSingleSlice,
  createStoreInstanceUsingSliceReducer,
} from '@/utils/createStoreInstanceUsingSliceReducer';
import { mockNetworkResponse } from '@/utils/mockNetworkResponse';
import { HttpStatusCode } from 'axios';
import {
  compartmentPathwaysDetailsFixture,
  compartmentPathwaysFixture,
  compartmentPathwaysOverLimitFixture,
} from '@/models/fixtures/compartmentPathways';
import { getModelsIds } from '@/components/Map/Drawer/ExportDrawer/ExportDrawer.component.utils';
import { unwrapResult } from '@reduxjs/toolkit';
import { MAIN_MAP_ID } from '@/constants/mocks';
import { apiPath } from '../apiPath';
import compartmentPathwaysReducer from './compartmentPathways.slice';
import { CompartmentPathwaysState } from './compartmentPathways.types';
import { getCompartmentPathways } from './compartmentPathways.thunks';
import { MODELS_MOCK } from './compartmentPathways.mock';

const mockedAxiosClient = mockNetworkResponse();
const MODELS_MOCK_IDS = getModelsIds(MODELS_MOCK);
const INITIAL_STATE: CompartmentPathwaysState = {
  loading: 'idle',
  error: { name: '', message: '' },
  data: [],
};

describe('compartmentPathways reducer', () => {
  let store = {} as ToolkitStoreWithSingleSlice<CompartmentPathwaysState>;
  beforeEach(() => {
    store = createStoreInstanceUsingSliceReducer('compartmentPathways', compartmentPathwaysReducer);
  });

  it('should match initial state', () => {
    const action = { type: 'unknown' };
    expect(compartmentPathwaysReducer(undefined, action)).toEqual(INITIAL_STATE);
  });
  it('should update store on loading getCompartmentPathways query', async () => {
    mockedAxiosClient
      .onGet(apiPath.getCompartmentPathwaysIds(52))
      .reply(HttpStatusCode.Ok, compartmentPathwaysFixture);
    mockedAxiosClient
      .onGet(apiPath.getCompartmentPathwayDetails([1, 2, 3]))
      .reply(HttpStatusCode.Ok, compartmentPathwaysDetailsFixture);
    mockedAxiosClient
      .onPost(apiPath.sendCompartmentPathwaysIds())
      .reply(HttpStatusCode.Ok, compartmentPathwaysDetailsFixture);

    const { loading, data } = store.getState().compartmentPathways;

    expect(loading).toEqual('idle');
    expect(data).toEqual([]);

    store.dispatch(getCompartmentPathways());

    const { loading: loadingPending, data: dataPending } = store.getState().compartmentPathways;

    expect(loadingPending).toEqual('pending');
    expect(dataPending).toEqual([]);
  });

  it('should update store after succesful getCompartmentPathways query', async () => {
    mockedAxiosClient
      .onGet(apiPath.getCompartmentPathwaysIds(MAIN_MAP_ID))
      .reply(HttpStatusCode.Ok, compartmentPathwaysFixture);
    mockedAxiosClient
      .onGet(apiPath.getCompartmentPathwaysIds(5054))
      .reply(HttpStatusCode.Ok, compartmentPathwaysOverLimitFixture);

    const ids = compartmentPathwaysFixture.map(el => el.id);
    mockedAxiosClient
      .onGet(apiPath.getCompartmentPathwayDetails(ids))
      .reply(HttpStatusCode.Ok, compartmentPathwaysDetailsFixture);
    mockedAxiosClient
      .onPost(apiPath.sendCompartmentPathwaysIds())
      .reply(HttpStatusCode.Ok, compartmentPathwaysDetailsFixture);

    const compartmentPathwaysPromise = store.dispatch(getCompartmentPathways(MODELS_MOCK_IDS));

    const { loading, data } = store.getState().compartmentPathways;

    expect(loading).toEqual('pending');
    expect(data).toEqual([]);

    const { type } = await compartmentPathwaysPromise;

    expect(type).toBe('compartmentPathways/getCompartmentPathways/fulfilled');

    const { loading: promiseFulfilled, data: dataFulfilled } = store.getState().compartmentPathways;

    expect(dataFulfilled).toEqual([
      ...compartmentPathwaysDetailsFixture,
      ...compartmentPathwaysDetailsFixture,
    ]);
    expect(promiseFulfilled).toEqual('succeeded');
  });

  it('should update store after failed getCompartmentPathways query', async () => {
    mockedAxiosClient
      .onGet(apiPath.getCompartmentPathwaysIds(MAIN_MAP_ID))
      .reply(HttpStatusCode.NotFound, []);
    mockedAxiosClient
      .onGet(apiPath.getCompartmentPathwayDetails([]))
      .reply(HttpStatusCode.NotFound, []);
    mockedAxiosClient
      .onPost(apiPath.sendCompartmentPathwaysIds())
      .reply(HttpStatusCode.NotFound, []);

    const compartmentPathwaysPromise = store.dispatch(getCompartmentPathways(MODELS_MOCK_IDS));

    const { loading, data } = store.getState().compartmentPathways;
    expect(loading).toEqual('pending');
    expect(data).toEqual([]);

    const dispatchData = await compartmentPathwaysPromise;

    expect(() => unwrapResult(dispatchData)).toThrow(
      "Failed to fetch compartment pathways: The page you're looking for doesn't exist. Please verify the URL and try again.",
    );
    const { loading: promiseFulfilled, data: dataFulfilled } = store.getState().compartmentPathways;

    expect(promiseFulfilled).toEqual('failed');
    expect(dataFulfilled).toEqual([]);
  });
});
